# Vulnerability-assessment-tool-admin chart

## Introduction
Helm Chart for vulnerability-assessment-tool cluster admin services

## Prerequisites
-   Kubernetes >=v1.15.0 with Beta APIs enabled
-   Persistent volume provisioner support in underlying infrastructure

## Installing the chart
To install the chart with the release name `release`:
```console
# With helm 2
# helm install { custom name } .
$ helm install admin-release .

# With helm 3, you need to create the desired namespace before hand
$ kubectl create namespace { namespace }
$ kubectl install admin-release .
```

The command deploys the vulnerability-assessment-tool-admin subchart of the vulnerability assessment tool v0.1.0
on the Kubernetes cluster in the default configuration. The configuration section lists
the parameters that can be configured during installation.

## Uninstalling the chart
To uninstall/delete the `release` deployment:
```console
$ helm delete admin-release
```

## Configuration
The following table lists the configurable parameters of the vulnerability-assessment-tool-admin chart and their default values.

| Parameter                                         | Description                                                                                                           | Default                                    |
| ------------------------------------------------- | --------------------------------------------------------------------------------------------------------------------- | ------------------------------------------ |
| global.namespace                                  | destination namespace for chart                                                                                       | `vulnerability-assessment-tool-monitoring` |
| global.coreNamespace                              | core chart namespace                                                                                                  | `vulnerability-assessment-tool-core`       |
| global.env                                        | label for deployment environment` | `TESTING`                                                                         |                                            |
| global.managedRelease                             | managed release name. This should correspond to the release name of the vulnerability-assessment-tool-core helm chart | `core-release`                                   |
| global.subPath                                    | subpath to serve from                                                                                                 | `""`                                       |
| global.ingressController.ports.http.port          |                                                                                                                       | `80`                                       |
| global.ingressController.ports.http.protocol      |                                                                                                                       | `TCP`                                      |
| global.ingressController.ports.http.targetPort    |                                                                                                                       | `80`                                       |
| global.ingressController.ports.https.port         |                                                                                                                       | `433`                                      |
| global.ingressController.ports.https.protocol     |                                                                                                                       | `TCP`                                      |
| global.ingressController.ports.https.targetPort   |                                                                                                                       | `433`                                      |
| global.ingressController.ports.metrics.port       |                                                                                                                       | `10254`                                    |
| global.ingressController.ports.metrics.protocol   |                                                                                                                       | `TCP`                                      |
| global.ingressController.ports.metrics.targetPort |                                                                                                                       | `10254`                                    |


### NGINX-Ingress controller subchart values
The subchart values can be configured by modifying the values directly in the subdirectory (for more configuration detail see [here](charts/nginx-ingress/README.md)) or overridden in the vulnerability-assessment-tool-admin chart by prepending values with `nginx-ingress`. For example, to modify the `managedRelease` value, you can do;
```
nginx-ingress:
  managedRelease: "canary"
```

## Production configuration
Every subchart contains a `values_production.yaml` file where you can find some parameters oriented to production configuration in comparison to the regular `values.yaml`. Since helm does not yet support recursive values file name, the switch can be done with this command:

```sh
# Backup default values.yaml file into values_simplified.yaml
for file in $(find . -mindepth 2 -type f -name "values.yaml"); do
  new=$(echo $file | sed -e 's/values.yaml/values_simplified.yaml/')
  mv "$file" "$new"
done

# Rename values_production.yaml to values.yaml
for file in $(find . -mindepth 2 -type f -name "values_production.yaml"); do
  new=$(echo $file | sed -e 's/values_production.yaml/values.yaml/')
  mv "$file" "$new"
done
```

To revert this operation simply do:
```sh
# Rename values.yaml file into values_production.yaml
for file in $(find . -mindepth 2 -type f -name "values.yaml"); do
  new=$(echo $file | sed -e 's/values.yaml/values_production.yaml/')
  mv "$file" "$new"
done

# Rename values_simplified.yaml to values.yaml
for file in $(find . -mindepth 2 -type f -name "values_simplified.yaml"); do
  new=$(echo $file | sed -e 's/values_simplified.yaml/values.yaml/')
  mv "$file" "$new"
done
```
