# Nginx-ingress: entrypoint for the vulnerability assessment tool chart

> This chart is already included in the vulnerability-assessment-tool-admin chart

This chart is a modified version of the stable nginx-ingress chart (see [chart-source](https://github.com/helm/charts/tree/master/stable/nginx-ingress)) with labeling scheme changes, some overall simplification, changing the google_containers/defaultbackend to an nginx container and made to serve only one release of the vulnerability-assessment-tool-core chart at a time (this restriction is soft and can be overcome simply by adding other ingresses).

## Prerequisites
-   Kubernetes >=v1.15.0 with Beta APIs enabled
-   Persistent volume provisioner support in underlying infrastructure

## Installing the chart
To install the chart with the release name `release`:
```console
$ helm install { custom release name } .
```

The command deploys the nginx-ingress subchart of the vulnerability-assessment-tool-admin v0.1.0
on the Kubernetes cluster in the default configuration. The configuration section lists
the parameters that can be configured during installation.

## Uninstalling the chart
To uninstall/delete the `release` deployment:
```console
$ helm delete { custom release name }
```

## Configuration
The following table lists the configurable parameters of the nginx-ingress chart and their default values. As values in this chart are quite highly nested, this table is split into three parts for easy readability

| Parameter      | Description                                                                         | Default |
| -------------- | ----------------------------------------------------------------------------------- | ------- |
| managedRelease | Sets the release name of the vulnerability assessment tool core chart to be managed | `alpha` |
| rbac.create    | If set to True create the required rbac for the ingress controller to function      | `True`  |
| ssl.enabled    | enables the reading the tls.crt and tls.key found in the files/ts                   | `False`  |

### .Values.defaultBackend
| Parameter | Description | Default |
| --- | --- | --- |
| enabled | bool to indicate whether or not to create default backend | `True` |
| replicas | Desired default proxy replicas | `1` |
| image.pullPolicy | This image is stable and quite simple so `IfNotPresent` is recommended | `IfNotPresent` |
| image.registry |  | `{}` |
| image.name |  | `nginx` |
| image.tag | | `1.17.4-alpine` |
| image.runAsUser | UID | `65534` |
| livenessProbe |  | enabled: `True`<br>initialDelaySeconds: `2`<br>periodSeconds: `10`<br>timeoutSeconds: `3`<br>failureThreshold: `5` |
| readinessProbe |  | enabled: `True`<br>initialDelaySeconds: `0`<br>periodSeconds: `5`<br>timeoutSeconds: `5`<br>failureThreshold: `6` |


### .Values.ingressController
| Parameter | Description | Default |
| --- | --- | --- |
| authIngress.enabled | enables basic auth ingress for `frontendbugs`, `kibana` as well as `prometheus` | `True` |
| authIngress.credentials.user | credentials are transfered in the clear if TLS is not instrumented due to NGINX's basic auth method  | `vulas` |
| authIngress.credentials.password |  | `changeme` |
| replicas | Desired nginx ingress controller deployment replicas | `2` |
| debugLevel | sets logging level for NGINX (*note*: access logs are disabled by default) | `error` |
| external | Creates a service for external LoadBalancers (should be compatible with all cloud providers) | `True` |
| electionID | Election ID to use for status update | `ingress-controller-leader` |
| ingressClass | Name of the ingress class to route through this controller | `nginx` |
| rbac | enables rbac | `True` |
| image.pullPolicy | Image maintained by kubernetes so `IfNotPresent` is the recommended pull policy | `IfNotPresent` |
| image.registry | | `quay.io` |
| image.name |  | `kubernetes-ingress-controller/nginx-ingress-controller` |
| image.tag | image tag | `0.25.0` |
| image.allowPrivilegeEscalation |  | `True` |
| livenessProbe | | enabled: `True`<br>initialDelaySeconds: `15`<br>periodSeconds: `10`<br>timeoutSeconds: `5`<br>failureThreshold: `10`<br>successThreshold: `1` |
| readinessProbe | | enabled: `True`<br>initialDelaySeconds: `10`<br>periodSeconds: `60`<br>timeoutSeconds: `5`<br>failureThreshold: `10`<br>successThreshold: `2`<br>path: `healthz` |

Specify each parameter using the --set key=value\[,key=value\] argument to helm install. For example,
```sh
$ helm install { custom release name }  \
  --set ingressController.replicas=5 .
```
Alternatively, a YAML file that specifies the values for the parameters can be provided while installing the chart. For example,
```sh
$ helm install { custom release name }  -f values.yaml .
```

## Production configuration
This chart includes a `values_production.yaml` file where you can find some parameters oriented to production configuration in comparison to the regular `values.yaml`.
```sh
$ helm install { custom release name }  -f values_production.yaml .
```
These values can be configured as follows:

### .Values.defaultBackend
| Parameter | Description | Default |
| --- | --- | --- |
| selfAntiAffinity | Makes defaultBackend's avoid sharing nodes with each other. Overkill, so can be disabled with `{}` | soft: `True`<br>weight: `100` |
| podDisruptionBudget.minAvailable | ensure availability during disruption | `1` |
| image.resources | These containers are not meant to be on high loads (serving only for redirects and static json for alerts) | requests:<br>&emsp;memory:`50Mi`<br>&emsp;cpu: `100m`<br>limit:<br>&emsp;memory: `50Mi`<br>&emsp;cpu: `100m` |


### .Values.ingressController
| Parameter | Description | Default |
| --- | --- | --- |
| selfAntiAffinity | Makes avoid sharing nodes with each other | soft: `True`<br>weight: `100` |
| podDisruptionBudget.minAvailable | ensure availability during disruption | `1` |
| persistentVolume.storage | storage size for PVC, useful for caching | `1Gi` |
| persistentVolume.storageG | storage size for PVC in NGINX compatible unit | `1g` |
| persistentVolume.mountPath | PVC mountpath | `/tmp/nginx-cache/` |
| persistentVolume.cache.enabled | Sets up the controller to serve static cache | `True` |

### SSL certificate
In order to serve securely, one can generate a tls key pair and add them in the `charts/nginx-ingress/files/tls` directory (the key file as `tls.crt` and `tls.key`, see [tls secret docs](https://kubernetes.io/docs/concepts/services-networking/ingress/)) and enable them with `ssl.enabled=True`. It is highly recommended to serve the vulnerability-assessment-tool with https.
