{{/*---------------------------- Local Templates ----------------------------*/}}
{{/*
  nginx-ingress.name : Gets name of current component
  contexts: [ . ]
  usage: {{ include "nginx-ingress.name" . }}
*/}}
{{- define "nginx-ingress.name" -}}
{{ .Release.Name }}-{{- default .Chart.Name | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/*
  nginx-ingress.chart : Create chart name and version as used by the chart label.
  contexts: [ . ]
  usage: {{ include "nginx-ingress.chart" . }}
*/}}
{{- define "nginx-ingress.chart" -}}
{{- printf "%s-%s-%s" .Chart.Name .Values.app.name .Chart.Version | replace "+" "_" | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/*
  nginx-ingress.tls.secretName : tls secret name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.tls.secretName" . }}
*/}}
{{- define "nginx-ingress.tls.secretName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "%s-tls-secret" $name -}}
{{- end -}}

{{/*
  nginx-ingress.clusterRoleName : cluster role name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.clusterRoleName" . }}
*/}}
{{- define "nginx-ingress.clusterRoleName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "%s-%s-ingress-controller" .Values.global.projectName $name -}}
{{- end -}}

{{/*
  nginx-ingress.clusterRoleBindingName : cluster role name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.clusterRoleBindingName" . }}
*/}}
{{- define "nginx-ingress.clusterRoleBindingName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "%s-%s-ingress-controller" .Values.global.projectName $name -}}
{{- end -}}

{{/*
  nginx-ingress.roleName : role name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.roleName" . }}
*/}}
{{- define "nginx-ingress.roleName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "%s-%s-%s-scoped-ingress-controller" .Values.global.projectName .Values.global.namespace $name -}}
{{- end -}}

{{/*
  nginx-ingress.roleBindingName : role binding name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.roleBindingName" . }}
*/}}
{{- define "nginx-ingress.roleBindingName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "%s-%s-%s-scoped-ingress-controller" .Values.global.projectName .Values.global.namespace $name -}}
{{- end -}}

{{/*
  nginx-ingress.redis.ingressName : ingress name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.redis.ingressName" . }}
*/}}
{{- define "nginx-ingress.redis.ingressName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "00-%s-redis-ingress" $name -}}
{{- end -}}

{{/*
  nginx-ingress.main.ingressName : ingress name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.main.ingressName" . }}
*/}}
{{- define "nginx-ingress.main.ingressName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "01-%s-main-ingress" $name -}}
{{- end -}}

{{/*
  nginx-ingress.auth.ingressName : ingress name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.auth.ingressName" . }}
*/}}
{{- define "nginx-ingress.auth.ingressName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "02-%s-auth-ingress" $name -}}
{{- end -}}

{{/*
  nginx-ingress.redirect.ingressName : redirection ingress name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.redirect.ingressName " . }}
*/}}
{{- define "nginx-ingress.redirect.ingressName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "07-%s-redirect-ingress" $name -}}
{{- end -}}

{{/*
  nginx-ingress.serviceAccountName : service account name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.serviceAccountName" . }}
*/}}
{{- define "nginx-ingress.serviceAccountName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "%s-ingress-controller" $name -}}
{{- end -}}

{{/*
  nginx-ingress.pvName : pv name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.pvName" . }}
*/}}
{{- define "nginx-ingress.pvName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "%s-persistent-volume" $name -}}
{{- end -}}

{{/*
  nginx-ingress.pvcName : pvc name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.pvcName" . }}
*/}}
{{- define "nginx-ingress.pvcName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "%s-persistent-volume-claim" $name -}}
{{- end -}}

{{/*
  nginx-ingress.redirect.serviceName : redirection service name getter
  contexts: [ . ]
  usage: {{ include "nginx-ingress.redirect.serviceName " . }}
*/}}
{{- define "nginx-ingress.redirect.serviceName" -}}
{{- $name := include "nginx-ingress.name" . -}}
{{- printf "%s-redirect-service" $name -}}
{{- end -}}




{{/*---------------------------- Global Templates ---------------------------*/}}
