# Vulnerability-assessment-tool-core

The Vulnerability-assessment-tool-core contains everything required to have a functional instance of the vulnerability-assessment-tool up and running.

## Prerequisites
-   Kubernetes >=v1.15.0 with Beta APIs enabled
-   Persistent volume provisioner support in underlying infrastructure

## Installing the chart
To install the chart with the release name `release`:
```console
# With helm 2
# helm install { custom name } .
$ helm install core-release .

# With helm 3, you need to create the desired namespace before hand
$ kubectl create namespace { namespace }
$ kubectl install core-release .
```

The command deploys the vulnerability-assessment-tool-core v0.1.1 chart on the Kubernetes cluster in the default configuration. The configuration section lists the parameters that can be configured during installation.

## Uninstalling the chart
To uninstall/delete the `release` deployment:
```console
$ helm delete core-release
```

## Configuration
The following table lists the configurable parameters of the vulnerability-assessment-tool-core chart and their default values.

| Parameter                             | Description                                               | Default                              |
| ------------------------------------- | --------------------------------------------------------- | ------------------------------------ |
| global.namespace                      | destination namespace for chart                           | `vulnerability-assessment-tool-core` |
| global.dbCredentials.pool_password    | pgpool password                                           | `changeme`                           |
| global.dbCredentials.pool_user        | pgpool username                                           | `pgpool`                             |
| global.dbCredentials.postgres_db      | postgres dbname                                           | `vulas`                              |
| global.dbCredentials.postgres_user    | postgres username                                         | `postgres`                           |
| global.dbCredentials.replication_user | replication username (shares password with postgres user) | `changeme`                           |
| global.env                            | label for deployment environment                          | `TESTING`                            |
| global.existingClaim                  | existingPVCClaim for master                               | `{}`                                 |
| global.masterReplicas                 | amount of masters in cluster                              | `1`                                  |
| global.podPriorityClass.enabled       | enables built in priorityclasses                          | `true`                               |
| global.projectName                    |                                                           | `vulnerability-assessment-tool`      |
| global.slaveReplicas                  | amount of slave replicas (can be set to {} for dynamic)   | `2`                                  |


### Frontendapps subchart values
The subchart values can be configured by modifying the values directly in the subdirectory (for more configuration detail see [here](charts/frontendapps/README.md) or overridden in the vulnerability-assessment-tool-admin chart by nesting values inside `frontendapps`. For example, to modify the `replicas` value, you can do;
```
frontendapps:
  replicas: 20
```

### Frontendbugs subchart values
These values should be prepended by nesting them in `frontendbugs` and can be seen in detail [here](charts/frontendbugs/README.md)

### Patchlibanalyzer subchart values
These values should be prepended by nesting them in `patchlibanalyzer` and can be seen in detail [here](charts/patchlibanalyzer/README.md)

### Database subchart values
These values should be prepended by nesting them in `database` and can be seen in detail [here](charts/database/README.md)

### Restbackend subchart values
These values should be prepended by nesting them in `restbackend` and can be seen in detail [here](charts/restbackend/README.md)

### Restlibutils subchart values
These values should be prepended by nesting them in `restlibutils` and can be seen in detail [here](charts/restlibutils/README.md)

## Production configuration
Every subchart contains a `values_production.yaml` file where you can find some parameters oriented to production configuration in comparison to the regular `values.yaml`. Since helm does not yet support recursive values file name, the switch can be done with this command:

```sh
# Backup default values.yaml file into values_simplified.yaml
for file in $(find . -mindepth 2 -type f -name "values.yaml"); do
  new=$(echo $file | sed -e 's/values.yaml/values_simplified.yaml/')
  mv "$file" "$new"
done

# Rename values_production.yaml to values.yaml
for file in $(find . -mindepth 2 -type f -name "values_production.yaml"); do
  new=$(echo $file | sed -e 's/values_production.yaml/values.yaml/')
  mv "$file" "$new"
done
```

To revert this operation simply do:
```sh
# Rename values.yaml file into values_production.yaml
for file in $(find . -mindepth 2 -type f -name "values.yaml"); do
  new=$(echo $file | sed -e 's/values.yaml/values_production.yaml/')
  mv "$file" "$new"
done

# Rename values_simplified.yaml to values.yaml
for file in $(find . -mindepth 2 -type f -name "values_simplified.yaml"); do
  new=$(echo $file | sed -e 's/values_simplified.yaml/values.yaml/')
  mv "$file" "$new"
done
```
