# Frontend-apps: an UI5 interface for managing bugs

> This chart is already included in the vulnerability-assessment-tool-core chart

## Prerequisites
-   Kubernetes >=v1.15.0 with Beta APIs enabled
-   Persistent volume provisioner support in underlying infrastructure

## Installing the chart
To install the chart with the release name `release`:
```console
$ helm install release .
```

The command deploys the frontendbugs subchart of the vulnerability-assessment-tool-core chart v0.1.1 chart
on the Kubernetes cluster in the default configuration. The configuration section lists
the parameters that can be configured during installation.

## Uninstalling the chart
To uninstall/delete the `release` deployment:
```console
$ helm delete release
```

## Configuration
The following table lists the configurable parameters of the frontendbugs chart and their default values.

| Parameter | Description | Default |
| --- | --- | --- |
| replicas | For HA `2` or `3` replicas are recommended | `2` |
| debug | Sets the debug level for scripts | `true` |
| horizontalPodAutoScaler.enabled | Enables an autoscaling (requires a metrics server up and running) | `False` |
| image.pullPolicy | These images are maintained by the team and won't be subject to drift | `IfNotPresent` |
| image.name |  | `vulas/vulnerability-assessment-tool-frontend-bugs` |
| image.tag |  | `3.1.7-SNAPSHOT-jib` |
| image.livenessProbe |  | enabled: `true`<br>initialDelaySeconds: `8`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `3` |
| image.readinessProbe |  | enabled: `true`<br>initialDelaySeconds: `5`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `3` |


## Production configuration
This chart includes a `values_production.yaml` file where you can find some parameters oriented to production configuration in comparison to the regular `values.yaml`.
```sh
$ helm install { custom release name }  -f values_production.yaml .
```
These values can be configured as follows:

| Parameter | Description | Default |
| --- | --- | --- |
| selfAntiAffinity | Makes frontend apps pod avoid each other | soft: `true`<br>weight: `100` |
| image.resources | The computing is done client-side so the requirements are quite low | requests:<br>&emsp;memory:`100Mi`<br>&emsp;cpu: `100m`<br>limit:<br>&emsp;memory: `200Mi`<br>&emsp;cpu: `200m` |
| image.securityContext |  | runAsUser: `65534`<br>runAsGroup: `65534`<br>privileged: `False`<br>readOnlyRootFilesystem: `False`<br>capabilities:<br>&emsp;drop: `["ALL"]` <br>&emsp;add: `["SYS_TIME", "NET_ADMIN"]` |

-   Disabling probres:

```diff
- livenessProbe.enabled: true
- readinessProbe.enabled: true
+ livenessProbe.enabled: False
+ readinessProbe.enabled: False
```
