# Restbackend

> This chart is already included in the vulnerability-assessment-tool-core chart

## Prerequisites
-   Kubernetes >=v1.15.0 with Beta APIs enabled
-   Persistent volume provisioner support in underlying infrastructure

## Installing the chart
To install the chart with the release name `release`:
```console
$ helm install release .
```

The command deploys the restbackend subchart of the vulnerability-assessment-tool-core chart v0.1.1 chart
on the Kubernetes cluster in the default configuration. The configuration section lists
the parameters that can be configured during installation.

## Uninstalling the chart
To uninstall/delete the `release` deployment:
```console
$ helm delete release
```

## Configuration
The following table lists the configurable parameters of the restbackend chart and their default values.

| Parameter | Description | Default |
| --- | --- | --- |
| replicas | For HA `2` or `3` replicas are recommended (only applied if set to deployment) | `3` |
| licenseConfidenceThreshold |  | `0.2` |
| langConfidenceThreshold |  | `0.2` |
| backoffDuration | delay between probe attempts | `5` |
| debug | Sets the debug level for scripts | `true` |
| rbac.create | Creates the rbac required for the restbackend to query the kube api (used for checking if all replicas are up) | `true` |
| horizontalPodAutoScaler.enabled | Enables autoscaling (requires a metric server) | `False` |
| podDisruptionBudget.minAvailable | Ensure availability during disruption | `1` |
| livenessProbe | | enabled: `true`<br>initialDelaySeconds: `30`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `15` |
| readinessProbe | | enabled: `true`<br>initialDelaySeconds: `30`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `15` |
| selfAntiAffinity.soft | Makes restbackend containers avoid sharing pods | `true` |
| selfAntiAffinity.weight | weight for said antiaffinity | `100` |
| image.initContainer.pullPolicy |  | `IfNotPresent` |
| image.initContainer.name |  | `postgres` |
| image.initContainer.tag | image tag | `11.3-alpine` |
| image.mainContainer.pullPolicy | These images are maintained by the team and won't be subject to drift | `IfNotPresent` |
| image.mainContainer.name |  | `vulas/vulnerability-assessment-tool-rest-backend` |
| image.mainContainer.tag | image tag | `3.1.7-SNAPSHOT-jib` |

## Production configuration
This chart includes a `values_production.yaml` file where you can find some parameters oriented to production configuration in comparison to the regular `values.yaml`.
```sh
$ helm install { custom release name }  -f values_production.yaml .
```
These values can be configured as follows:

| Parameter | Description | Default |
| --- | --- | --- |
| image.initContainer.securityContext |  | runAsUser: `65534`<br>runAsGroup: `65534`<br>privileged: `False`<br>readOnlyRootFilesystem: `False`<br>capabilities:<br>&emsp;drop: `["ALL"]` <br>&emsp;add: `["SYS_TIME", "NET_ADMIN"]` |
| image.initContainer.resources |  | requests:<br>&emsp;memory:`25Mi`<br>&emsp;cpu: `100m`<br>limit:<br>&emsp;memory: `35Mi`<br>&emsp;cpu: `200m` |
| image.mainContainer.securityContext |  | runAsUser: `0`<br>privileged: `False`<br>readOnlyRootFilesystem: `False` |
| image.mainContainer.resources | The restbackend consumes quite a bit of resources. These values are adpated for production values with around 300GB of data in the db | requests:<br>&emsp;memory:`8Gi`<br>&emsp;cpu: `4`<br>limit:<br>&emsp;memory: `16Gi`<br>&emsp;cpu: `8` |
