# Restlibutils

> This chart is already included in the vulnerability-assessment-tool-core chart

## Prerequisites
-   Kubernetes >=v1.15.0 with Beta APIs enabled
-   Persistent volume provisioner support in underlying infrastructure

## Installing the chart
To install the chart with the release name `release`:
```console
$ helm install release .
```

The command deploys the restlibutils subchart of the vulnerability-assessment-tool-core chart v0.1.1 chart
on the Kubernetes cluster in the default configuration. The configuration section lists
the parameters that can be configured during installation.

## Uninstalling the chart
To uninstall/delete the `release` deployment:
```console
$ helm delete release
```

## Configuration
The following table lists the configurable parameters of the restlibutils chart and their default values.


| Parameter | Description | Default |
| --- | --- | --- |
| replicas | For HA `2` or `3` replicas are recommended (only applied if set to deployment) | `3` |
| licenseConfidenceThreshold |  | `0.2` |
| langConfidenceThreshold |  | `0.2` |
| backoffDuration | delay between probe attempts | `5` |
| debug | Sets the debug level for scripts | `true` |
| rbac.create | Creates the rbac required for the restlibutils to query the kube api (used for checking if all replicas are up) | `true` |
| horizontalPodAutoScaler.enabled | Enables autoscaling (requires a metric server) | `False` |
| podDisruptionBudget.minAvailable | Ensure availability ²during disruption | `1` |
| livenessProbe | | enabled: `true`<br>initialDelaySeconds: `25`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `5` |
| readinessProbe | | enabled: `true`<br>initialDelaySeconds: `20`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `5` |
| image.pullPolicy | These images are maintained by the team and won't be subject to drift | `IfNotPresent` |
| image.name |  | `vulas/vulnerability-assessment-tool-rest-backend` |
| image.tag | | `3.1.7-SNAPSHOT-jib` |

## Production configuration
This chart includes a `values_production.yaml` file where you can find some parameters oriented to production configuration in comparison to the regular `values.yaml`.
```sh
$ helm install { custom release name }  -f values_production.yaml .
```
These values can be configured as follows:

| Parameter | Description | Default |
| --- | --- | --- |
| persistentVolume.storage |  | `2Gi` |
| persistentVolume.mountPath | PVC mountpath | `/.m2` |
| image.mainContainer.securityContext |  | runAsUser: `65534`<br>runAsGroup: `65534`<br>privileged: `False`<br>readOnlyRootFilesystem: `False` |
| image.mainContainer.resources | The restlibutils consumes quite a bit of resources. These values are adapted for production values with around 300GB of data in the db | requests:<br>&emsp;memory:`8Gi`<br>&emsp;cpu: `2`<br>limit:<br>&emsp;memory: `15Gi`<br>&emsp;cpu: `7` |
| selfAntiAffinity.soft | Makes reslibutils containers avoid sharing pods | `true` |
| selfAntiAffinity.weight | weight for said antiaffinity | `100` |
