{{/*---------------------------- Local Templates ----------------------------*/}}
{{/*
  elastic.name : Gets name of current component
  contexts: [ . ]
  usage: {{ include "elastic.name" . }}
*/}}
{{- define "elastic.name" -}}
{{ .Release.Name }}-elasticsearch
{{- end -}}

{{/*
  elastic.clusterName : Gets name of current component
  contexts: [ . ]
  usage: {{ include "elastic.clusterName" . }}
*/}}
{{- define "elastic.clusterName" -}}
{{- $name := include "elastic.name" . -}}
{{- printf "%s-cluster" $name -}}
{{- end -}}

{{/*
  elastic.chart : Create chart name and version as used by the chart label.
  contexts: [ . ]
  usage: {{ include "patch.chart" . }}
*/}}
{{- define "elastic.chart" -}}
{{- printf "%s-%s" .Chart.Name .Chart.Version | replace "+" "_" | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/*
  elastic.configMapName : ConfigMap getter
  contexts: [ . ]
  usage: {{ include "elastic.configMapName" . }}
*/}}
{{- define "elastic.configMapName" -}}
{{- $name := include "elastic.name" . -}}
{{- printf "%s-config" $name -}}
{{- end -}}


{{/*
  elastic.statefulSetName : Deployment stateful set name getter
  contexts: [ . ]
  usage: {{ include "elastic.statefulSetName" . }}
*/}}
{{- define "elastic.statefulSetName" -}}
{{- $name := include "elastic.name" . -}}
{{- printf "%s-stateful-set" $name -}}
{{- end -}}


{{/*
  elastic.pvcName : elastic pvc name getter
  contexts: [ . ]
  usage: {{ include "elastic.pvcName" . }}
*/}}
{{- define "elastic.pvcName" -}}
{{- $name := include "elastic.name" . -}}
{{- printf "%s-volume-claim" $name -}}
{{- end -}}


{{/*
  elastic.pvName : elastic pv name getter
  contexts: [ . ]
  usage: {{ include "elastic.pvName" . }}
*/}}
{{- define "elastic.pvName" -}}
{{- $name := include "elastic.name" . -}}
{{- printf "%s-persistent-nfs-volume" $name -}}
{{- end -}}


{{/*
  elastic.podName : elastic pod name getter
  contexts: [ . ]
  usage: {{ include "elastic.podName" . }}
*/}}
{{- define "elastic.podName" -}}
{{- $name := include "elastic.name" . -}}
{{- printf "%s-pod" $name -}}
{{- end -}}


{{/*
  elastic.podDisruptionBudgetName : elastic pod disruption budget name getter
  contexts: [ . ]
  usage: {{ include "elastic.podDisruptionBudgetName" . }}
*/}}
{{- define "elastic.podDisruptionBudgetName" -}}
{{- $name := include "elastic.name" . -}}
{{- printf "%s-pod-disruption-budget" $name -}}
{{- end -}}



{{- define "elastic.endpoints" -}}
{{- $replicas := .Values.elasticsearch.replicas | int }}
{{- $name := include "elastic.statefulSetName" . -}}
{{- range $i, $e := untilStep 0 $replicas 1 -}}
{{ printf "%s-%d," $name $i }}
{{- end -}}
{{- end -}}

{{/*---------------------------- Global Templates ---------------------------*/}}
{{/*
  elastic.headlessServiceName : elastic service name getter
  contexts: [ .global ]
  usage: {{ include "elastic.headlessServiceName" . }}
*/}}
{{- define "elastic.headlessServiceName" -}}
{{- printf "%s-headless" .Values.global.monitoring.elasticsearch.service -}}
{{- end -}}
