# Prometheus

> This chart is already included in the vulnerability-assessment-tool-monitoring chart

This chart is a modified version of the stable chart (see [source](https://github.com/helm/charts/tree/master/stable/prometheus)) with a lot of simplification and some labeling / name schema changes.

## Prerequisites
-   Kubernetes >=v1.15.0 with Beta APIs enabled
-   Persistent volume provisioner support in underlying infrastructure

## Installing the chart
To install the chart with the release name `release`:
```console
$ helm install release .
```

The command deploys the prometheus subchart of the vulnerability-assessment-tool-monitoring v0.1.1
on the Kubernetes cluster in the default configuration. The configuration section lists
the parameters that can be configured during installation.

## Uninstalling the chart
To uninstall/delete the `release` deployment:
```console
$ helm delete release
```

## Configuration
The following table lists the configurable parameters of the prometheus chart and their default values. As values in this chart are quite highly nested, this table is split into four parts for easy readability

| Parameter  |	Description  |	Default |
|----------|-------------|--------|
| rbac.create | bool to decide whether or not to add rbac layer | `True` |

### .Values.kubeStateMetrics
| Parameter | Description | Default |
| --- |--- | --- |
| enabled | Enables the kubeStateMetrics deployment | `True` |
| replicas | One instance should be plenty for a small cluster if Fault tolerance is not the goal | `1` |
| selfAntiAffinity | antiaffinity towards sharing a node with similar pods | soft: `True`<br>weight: `100` |
| image.pullPolicy |  | `IfNotPresent` |
| image.registry |  | `quay.io` |
| image.name |  | `coreos/kube-state-metrics` |
| image.tag |  | `v1.7.2` |
| image.livenessProbe |  | enabled: `False`<br>initialDelaySeconds: `5`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `5` |
| image.readinessProbe |  | enabled: `False`<br>initialDelaySeconds: `5`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `5` |
| image.resources |  | requests:<br>&emsp;memory: `32Mi`<br>&emsp;cpu: `10m`limit:<br>&emsp;memory: `64Mi`<br>&emsp;cpu: `40m` |

### .Values.nodeExporter
| Parameter | Description | Default |
| --- |--- | --- |
| enabled | Enables the nodeExporter daemonset | `True` |
| selfAntiAffinity | antiaffinity towards sharing a node with similar pods | soft: `True`<br>weight: `100` |
| image.pullPolicy || `IfNotPresent` |
| image.name |  | `prom/node-exporter` |
| image.tag |  | `v0.18.1` |
| image.livenessProbe |  | enabled: `False`<br>initialDelaySeconds: `5`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `5` |
| image.readinessProbe |  | enabled: `False`<br>initialDelaySeconds: `5`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `5` |
| image.resources |  | requests:<br>&emsp;memory: `30Mi`<br>&emsp;cpu: `100m`limit:<br>&emsp;memory: `50Mi`<br>&emsp;cpu: `200m` |
| podSecurityPolicy.enabled |  | `True` |
| podSecurityPolicy.hostNetwork |  | `True` |
| podSecurityPolicy.hostPID |  | `True` |

### .Values.server (Prometheus Server)
| Parameter | Description | Default |
| --- | --- | --- |
| path | Path to serve the admin api for prometheus set by `--web.external-url` | `/monitoring/prometheus/` |
| replicas | One replica should suffice for an small cluster | `1` |
| podManagementPolicy |  | `Parallel` |
| enableAdminApi | Allows the admin API | `True` |
| retention | metrics retention duration | `90d` |
| selfAntiAffinity | antiaffinity towards sharing a node with similar pods | soft: `True`<br>weight: `100` |
| persistentVolume | pvc for metrics | enabled: `False`<br>mountPath: `/data`<br>storage: `2Gi` |
| image.mainContainer.pullPolicy |  | `IfNotPresent` |
| image.mainContainer.name |  | `prom/prometheus` |
| image.mainContainer.tag |  | `v2.12.0` |
| image.mainContainer.livenessProbe |  | enabled: `False`<br>initialDelaySeconds: `30`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `5`<br>path: `/healthy` |
| image.mainContainer.readinessProbe |  | enabled: `False`<br>initialDelaySeconds: `30`<br>periodSeconds: `30`<br>timeoutSeconds: `5`<br>failureThreshold: `5`<br>successThreshold: `1`<br>path: `/ready` |
| image.mainContainer0resources |  | requests:<br>&emsp;memory: `512Mi`<br>&emsp;cpu: `500m`limit:<br>&emsp;memory: `512Mi`<br>&emsp;cpu: `500m` |
